@@ -1,7 +1,8 @@
 import base64
 import os
 import thread
-from sqlobject.dbconnection import DBAPI
+import urllib
+from sqlobject.dbconnection import DBAPI, Boolean
 from sqlobject import col, sqlbuilder
 from sqlobject.dberrors import *
 
@@ -22,29 +23,29 @@ class SQLiteConnection(DBAPI):
     schemes = [dbName]
 
     def __init__(self, filename, autoCommit=1, **kw):
-        backends = kw.pop('backend', None) or 'pysqlite2,sqlite3,sqlite'
-        for backend in backends.split(','):
-            backend = backend.strip()
-            if not backend:
+        drivers = kw.pop('driver', None) or 'pysqlite2,sqlite3,sqlite'
+        for driver in drivers.split(','):
+            driver = driver.strip()
+            if not driver:
                 continue
             try:
-                if backend in ('sqlite2', 'pysqlite2'):
+                if driver in ('sqlite2', 'pysqlite2'):
                         from pysqlite2 import dbapi2 as sqlite
                         self.using_sqlite2 = True
-                elif backend == 'sqlite3':
+                elif driver == 'sqlite3':
                         import sqlite3 as sqlite
                         self.using_sqlite2 = True
-                elif backend in ('sqlite', 'sqlite1'):
+                elif driver in ('sqlite', 'sqlite1'):
                         import sqlite
                         self.using_sqlite2 = False
                 else:
-                    raise ValueError('Unknown SQLite backend "%s", expected pysqlite2, sqlite3 or sqlite' % backend)
+                    raise ValueError('Unknown SQLite driver "%s", expected pysqlite2, sqlite3 or sqlite' % driver)
             except ImportError:
                 pass
             else:
                 break
         else:
-            raise ImportError('Cannot find an SQLite backend, tried %s' % backends)
+            raise ImportError('Cannot find an SQLite driver, tried %s' % drivers)
         if self.using_sqlite2:
             sqlite.encode = base64.encodestring
             sqlite.decode = base64.decodestring
@@ -58,10 +59,6 @@ class SQLiteConnection(DBAPI):
         if self.using_sqlite2:
             if autoCommit:
                 opts["isolation_level"] = None
-            opts["detect_types"] = sqlite.PARSE_DECLTYPES
-            for col_type in "text", "char", "varchar", "date", "time", "datetime", "timestamp":
-                sqlite.register_converter(col_type, stop_pysqlite2_converting_strings)
-                sqlite.register_converter(col_type.upper(), stop_pysqlite2_converting_strings)
             global sqlite2_Binary
             if sqlite2_Binary is None:
                 sqlite2_Binary = sqlite.Binary
@@ -72,7 +69,7 @@ class SQLiteConnection(DBAPI):
                     factory = globals()[factory]
                 opts['factory'] = factory(sqlite)
         else:
-            opts['autocommit'] = bool(autoCommit)
+            opts['autocommit'] = Boolean(autoCommit)
             if 'encoding' in kw:
                 opts['encoding'] = kw.pop('encoding')
             if 'mode' in kw:
@@ -83,37 +80,52 @@ class SQLiteConnection(DBAPI):
             else:
                 opts['timeout'] = int(float(kw.pop('timeout')) * 1000)
         if 'check_same_thread' in kw:
-            opts["check_same_thread"] = bool(kw.pop('check_same_thread'))
+            opts["check_same_thread"] = Boolean(kw.pop('check_same_thread'))
         # use only one connection for sqlite - supports multiple)
         # cursors per connection
         self._connOptions = opts
-        self.use_table_info = kw.pop("use_table_info", False)
+        self.use_table_info = Boolean(kw.pop("use_table_info", True))
         DBAPI.__init__(self, **kw)
         self._threadPool = {}
         self._threadOrigination = {}
         if self._memory:
             self._memoryConn = sqlite.connect(
                 self.filename, **self._connOptions)
+            # Convert text data from SQLite to str, not unicode -
+            # SQLObject converts it to unicode itself.
+            self._memoryConn.text_factory = str
 
-    def connectionFromURI(cls, uri):
-        user, password, host, port, path, args = cls._parseURI(uri)
-        assert host is None, (
+    @classmethod
+    def _connectionFromParams(cls, user, password, host, port, path, args):
+        assert host is None and port is None, (
             "SQLite can only be used locally (with a URI like "
-            "sqlite:///file or sqlite:/file, not %r)" % uri)
+            "sqlite:/file or sqlite:///file, not sqlite://%s%s)" %
+            (host, port and ':%r' % port or ''))
         assert user is None and password is None, (
             "You may not provide usernames or passwords for SQLite "
             "databases")
         if path == "/:memory:":
             path = ":memory:"
         return cls(filename=path, **args)
-    connectionFromURI = classmethod(connectionFromURI)
+
+    def oldUri(self):
+        path = self.filename
+        if path == ":memory:":
+            path = "/:memory:"
+        else:
+            path = "//" + path
+        return 'sqlite:%s' % path
 
     def uri(self):
         path = self.filename
         if path == ":memory:":
             path = "/:memory:"
         else:
-            path = "//" + path
+            if path.startswith('/'):
+                path = "//" + path
+            else:
+                path = "///" + path
+            path = urllib.quote(path)
         return 'sqlite:%s' % path
 
     def getConnection(self):
@@ -127,7 +139,7 @@ class SQLiteConnection(DBAPI):
             return conn
         threadid = thread.get_ident()
         if (self._pool is not None
-            and self._threadPool.has_key(threadid)):
+            and threadid in self._threadPool):
             conn = self._threadPool[threadid]
             del self._threadPool[threadid]
             if conn in self._pool:
@@ -151,7 +163,7 @@ class SQLiteConnection(DBAPI):
         threadid = self._threadOrigination.get(id(conn))
         DBAPI.releaseConnection(self, conn, explicit=explicit)
         if (self._pool is not None and threadid
-            and not self._threadPool.has_key(threadid)):
+            and threadid not in self._threadPool):
             self._threadPool[threadid] = conn
         else:
             if self._pool and conn in self._pool:
@@ -175,7 +187,13 @@ class SQLiteConnection(DBAPI):
     def makeConnection(self):
         if self._memory:
             return self._memoryConn
-        return self.module.connect(self.filename, **self._connOptions)
+        conn = self.module.connect(self.filename, **self._connOptions)
+        conn.text_factory = str # Convert text data to str, not unicode
+        return conn
+
+    def close(self):
+        DBAPI.close(self)
+        self._threadPool = {}
 
     def _executeRetry(self, conn, cursor, query):
         if self.debug:
@@ -234,13 +252,13 @@ class SQLiteConnection(DBAPI):
         else:
             return DBAPI._insertSQL(self, table, names, values)
 
+    @classmethod
     def _queryAddLimitOffset(cls, query, start, end):
         if not start:
             return "%s LIMIT %i" % (query, end)
         if not end:
             return "%s LIMIT 0 OFFSET %i" % (query, start)
         return "%s LIMIT %i OFFSET %i" % (query, end-start, start)
-    _queryAddLimitOffset = classmethod(_queryAddLimitOffset)
 
     def createColumn(self, soClass, col):
         return col.sqliteCreateSQL()
@@ -252,8 +270,9 @@ class SQLiteConnection(DBAPI):
         return self._createIDColumn(soClass.sqlmeta)
 
     def _createIDColumn(self, sqlmeta):
-        key_type = {int: "INTEGER", str: "TEXT"}[sqlmeta.idType]
-        return '%s %s PRIMARY KEY' % (sqlmeta.idName, key_type)
+        if sqlmeta.idType == str:
+            return '%s TEXT PRIMARY KEY' % sqlmeta.idName
+        return '%s INTEGER PRIMARY KEY AUTOINCREMENT' % sqlmeta.idName
 
     def joinSQLType(self, join):
         return 'INT NOT NULL'
@@ -380,7 +399,3 @@ class SQLiteConnection(DBAPI):
         if self._memory:
             return
         os.unlink(self.filename)
-
-
-def stop_pysqlite2_converting_strings(s):
-    return s
